/*
 *  dvipr
 *      TeX Printer driver for The Canon LIPS III/IVc, CaPSL III printer
 *
 *      Developed at:
 *          Computer Systems Research Group
 *          Department of Information Science
 *          Fukui University
 *          9-1, Bunkyo-3, Fukui, 910 Japan
 *
 *      Authors:
 *          Yasumasa Koizumi, Atsushi Takagi, Kenichi Sugimoto,
 *          and Minato Kawaguti
 *
 *  The program may be copied and redistributed freely in the spirit of
 *  Free Software Foundation's "Copy Left", provided this title page is
 *  included in all of the source files involved.
 *
 *  Reports on porting to new platforms, bug fixes, etc., are highly
 *  appreciated. e-mail of this kind of communication shhould be addressedd
 *  to:
 *      dvipr@i1nws1.fukui-u.ac.jp
 */

#include <string.h>
#include "newtypes.h"

unsigned WORD JisToKuten(unsigned WORD jis)
{
	return jis - 0x2020;
}

unsigned WORD KutenToJis(unsigned WORD kuten)
{
	return kuten + 0x2020;
}

BOOL GetDnpFontInfoByKutenCode(unsigned WORD kuten, char **pFontPrefix, int *nCode, int *nFontID)
{
	int			ku, ten, code;
	static char	*pFontPrefixTmp = "k ";
	static char *apFontPrefixTable[8] = {"sy", "sy", "roma", "hira", "kata", "greek", "russian", "keisen"};
	
	ku = kuten >> 8;
	ten = kuten & 0xff;
	if (1 <= ku && ku <= 8) {
		*pFontPrefix = apFontPrefixTable[ku - 1];
		if (ku == 2)
			*nCode = ten + 0x64;
		else if (ku == 3)
			*nCode = ten + 0x20;
		else
			*nCode = ten;
		*nFontID = ((ku == 1) ? 1 : ku - 1);
		return TRUE;
	} else if (16 <= ku && ku <= 47) {
		code = (ku - 16) * 94 + ten - 1;
		*pFontPrefix = pFontPrefixTmp;
		pFontPrefixTmp[1] = 'a' + code / 256;
		*nCode = code % 256;
		*nFontID = code / 256 + 8;
		return TRUE;
	} else if (48 <= ku && ku <= 84) {
		code = (ku - 48) * 94 + ten - 1;
		*pFontPrefix = pFontPrefixTmp;
		pFontPrefixTmp[1] = 'm' + code / 256;
		*nCode = code % 256;
		*nFontID = code / 256 + 20;
		return TRUE;
	}
	return FALSE;
}

BOOL GetKutenCodeByDnpFontInfo(char *pFontPrefix, int nCode, unsigned WORD *kuten)
{
	int		code;
	
	if (!strncmp(pFontPrefix, "sy", 2))
		if (nCode < 0x60) {
			*kuten = 0x0100 + nCode;
			return TRUE;
		} else {
			*kuten = 0x0200 + (nCode - 0x64);
			return TRUE;
		}
	else if (!strncmp(pFontPrefix, "roma", 4)) {
		*kuten = 0x0310 + (nCode - 0x30);
		return TRUE;
	} else if (!strncmp(pFontPrefix, "hira", 4)) {
		*kuten = 0x0400 + nCode;
		return TRUE;
	} else if (!strncmp(pFontPrefix, "kata", 4)) {
		*kuten = 0x0500 + nCode;
		return TRUE;
	} else if (!strncmp(pFontPrefix, "greek", 5)) {
		*kuten = 0x0600 + nCode;
		return TRUE;
	} else if (!strncmp(pFontPrefix, "russian", 7)) {
		*kuten = 0x0700 + nCode;
		return TRUE;
	} else if (!strncmp(pFontPrefix, "keisen", 6)) {
		*kuten = 0x0800 + nCode;
		return TRUE;
	} else if (pFontPrefix[0] == 'k')
		if ('a' <= pFontPrefix[1] && pFontPrefix[1] <= 'l') {
			code = ((pFontPrefix[1] - 'a') << 8) + nCode;
			*kuten = ((code / 94 + 16) << 8) + ((code % 94) + 1);
			return TRUE;
		} else if ('m' <= pFontPrefix[1] && pFontPrefix[1] <= 'z') {
			code = ((pFontPrefix[1] - 'm') << 8) + nCode;
			*kuten = ((code / 94 + 48) << 8) + ((code % 94) + 1);
			return TRUE;
		}
	return FALSE;
}
