/*
 *  dvipr
 *      TeX Printer driver for The Canon LIPS III/IVc, CaPSL III printer
 *
 *      Developed at:
 *          Computer Systems Research Group
 *          Department of Information Science
 *          Fukui University
 *          9-1, Bunkyo-3, Fukui, 910 Japan
 *
 *      Authors:
 *          Yasumasa Koizumi, Atsushi Takagi, Kenichi Sugimoto,
 *          and Minato Kawaguti
 *
 *  The program may be copied and redistributed freely in the spirit of
 *  Free Software Foundation's "Copy Left", provided this title page is
 *  included in all of the source files involved.
 *
 *  Reports on porting to new platforms, bug fixes, etc., are highly
 *  appreciated. e-mail of this kind of communication shhould be addressedd
 *  to:
 *      dvipr@i1nws1.fukui-u.ac.jp
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "newtypes.h"
#include "error.h"
#include "gvars.h"
#include "pdl.h"
#include "funcdef.h"

void Error(ErrorCode err)
{
	char *ptr;

	ptr = SearchErrorMessage(err, ".err");
	if (ptr != NULL)
		fprintf(stderr, "%s\n", ptr);
	else
		fprintf(stderr, "Error file not found or unknown error [#%d]\n", err);
	if (err > 1000)
		gpfnPdlError(err);
}

void Error2(ErrorCode err, char* arg1)
{
	char *ptr;

	ptr = SearchErrorMessage(err, ".err");
	if (ptr != NULL) {
		fprintf(stderr, ptr, arg1);
		fprintf(stderr, "\n");
	} else
		fprintf(stderr, "Error file not found or unknown error [#%d]\n", err);
	gpfnPdlError(err);
}

char *SearchErrorMessage(ErrorCode err, char *pszFileExt)
{
	char aszErrorFileName[FILENAME_MAX], *ptr, *ret;
	FILE *fp;
	static char acBuff[BUFSIZ];

	ret = NULL;
	sprintf(aszErrorFileName, "%s/dvipdl%s", DVIPR_DATA_DIR, pszFileExt);
	if ((fp = fopen(aszErrorFileName, "r")) != NULL) {
		while (fgets(acBuff, BUFSIZ, fp) != NULL) {
			*strchr(acBuff, '\n') = '\0';
			if (acBuff[0] == '#')
				continue;
			if ((ptr = strtok(acBuff, "\t")) == NULL)
				continue;
			if (atoi(ptr) == err) {
				ret = strtok(NULL, "\t");
				break;
			}
		}
		fclose(fp);
	} else
		fprintf(stderr, "File '%s' can't open.\n", aszErrorFileName);
	return ret;
}
