/*
 *  dvipr
 *      TeX Printer driver for The Canon LIPS III/IVc, CaPSL III printer
 *
 *      Developed at:
 *          Computer Systems Research Group
 *          Department of Information Science
 *          Fukui University
 *          9-1, Bunkyo-3, Fukui, 910 Japan
 *
 *      Authors:
 *          Yasumasa Koizumi, Atsushi Takagi, Kenichi Sugimoto,
 *          and Minato Kawaguti
 *
 *  The program may be copied and redistributed freely in the spirit of
 *  Free Software Foundation's "Copy Left", provided this title page is
 *  included in all of the source files involved.
 *
 *  Reports on porting to new platforms, bug fixes, etc., are highly
 *  appreciated. e-mail of this kind of communication shhould be addressedd
 *  to:
 *      dvipr@i1nws1.fukui-u.ac.jp
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include "dvipr.h"
#include "dvipdl.h"
#include "site.h"
#include "error.h"
#include "file.h"
#include "funcdef.h"

#define	DVIPDL_COMMAND_MAX	256

int main(int argc, char *argv[])
{
	int		nOptionLength, nOptionsCount, ch, i;
	char	aszDvipdlOptions[DVIPDL_OPTIONS_MAX];
	char	*pszDvifileName;
	char	*apszArgv[DVIPDL_ARGUMENTITEM_MAX];
	FILE	*fp;

	if (FpRead4(stdin) != DVIPR_ID)
		return -1;
	nOptionLength = FpRead2(stdin);
	if (nOptionLength > DVIPDL_OPTIONS_MAX)
		return -2;
	fread(aszDvipdlOptions, nOptionLength, 1, stdin);
	if ((pszDvifileName = tmpnam(NULL)) == NULL)
		return -3;
	if ((fp = fopen(pszDvifileName, "w")) == NULL)
		return -4;
	while ((ch = fgetc(stdin)) != EOF)
		fputc(ch, fp);
	fclose(fp);
	apszArgv[0] = DVIPDL_CMD;
	apszArgv[1] = pszDvifileName;
	apszArgv[2] = strtok(aszDvipdlOptions, " ");
	for (nOptionsCount = 3; nOptionsCount < DVIPDL_ARGUMENTITEM_MAX; nOptionsCount++)
		if ((apszArgv[nOptionsCount] = strtok(NULL, " ")) == NULL)
			break;
	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
				case 'x':
					apszArgv[nOptionsCount++] = "-x";
					apszArgv[nOptionsCount++] = &argv[i][2];
					break;
				case 'y':
					apszArgv[nOptionsCount++] = "-y";
					apszArgv[nOptionsCount++] = &argv[i][2];
					break;
				case 'n':
					apszArgv[nOptionsCount++] = "-u";
					apszArgv[nOptionsCount++] = argv[++i];
					break;
				case 'h':
					apszArgv[nOptionsCount++] = "-U";
					apszArgv[nOptionsCount++] = argv[++i];
					break;
				default:
					fprintf(stderr, "Unknown option [%s]", argv[i]);
					break;
			}
		} else {
			apszArgv[nOptionsCount++] = "-a";
			apszArgv[nOptionsCount++] = argv[i];
		}
	}
	apszArgv[nOptionsCount++] = NULL;
	if (fork())
		wait(NULL);
	else
		execv(DVIPDL_CMD, apszArgv);
	unlink(pszDvifileName);
	return 0;
}

void FatalExit(int nExit)
{
	exit(nExit);
}
