/*
 *  dvipr
 *      TeX Printer driver for The Canon LIPS III/IVc, CaPSL III printer
 *
 *      Developed at:
 *          Computer Systems Research Group
 *          Department of Information Science
 *          Fukui University
 *          9-1, Bunkyo-3, Fukui, 910 Japan
 *
 *      Authors:
 *          Yasumasa Koizumi, Atsushi Takagi, Kenichi Sugimoto,
 *          and Minato Kawaguti
 *
 *  The program may be copied and redistributed freely in the spirit of
 *  Free Software Foundation's "Copy Left", provided this title page is
 *  included in all of the source files involved.
 *
 *  Reports on porting to new platforms, bug fixes, etc., are highly
 *  appreciated. e-mail of this kind of communication shhould be addressedd
 *  to:
 *      dvipr@i1nws1.fukui-u.ac.jp
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "error.h"
#include "dvipdl.h"
#include "file.h"
#include "funcdef.h"

static unsigned char cCache;
static BOOL bNibble = FALSE;

char FpReadNibble(FILE *fp)
{
	bNibble = !bNibble;
	if (bNibble)
		return (cCache = FpRead1(fp)) >> 4;
	else
		return cCache & 0x0f;
}

void FpResetNibble(void)
{
	bNibble = FALSE;
}

unsigned char FpRead1(FILE *fp)
{
	int	ret;

	if ((ret = fgetc(fp)) == EOF) {
		Error(ERR_EOF);
		FatalExit(-1);
	}
	return (unsigned char )ret;
}

WORD FpRead2(FILE *fp)
{
	return ((unsigned WORD)FpRead1(fp) << 8) | (unsigned WORD)(FpRead1(fp) & 0xff);
}

DWORD FpRead3(FILE *fp)
{
	return ((unsigned DWORD)FpRead2(fp) << 8) | (unsigned DWORD)(FpRead1(fp) & 0xff);
}

DWORD FpRead4(FILE *fp)
{
	return ((unsigned DWORD)FpRead2(fp) << 16) | (unsigned DWORD)(FpRead2(fp) & 0xffff);
}

void FpWrite1(FILE *fp, int ch)
{
	if (fputc(ch, fp) == EOF) {
		Error(ERR_EOF);
		FatalExit(-1);
	}
}

void FpWrite2(FILE *fp, WORD wParam)
{
	FpWrite1(fp, (wParam >> 8) & 0xff);
	FpWrite1(fp, wParam & 0xff);
}

void FpWrite3(FILE *fp, DWORD dwParam)
{
	FpWrite2(fp, (dwParam >> 8) & 0xffff);
	FpWrite1(fp, dwParam & 0xff);
}

void FpWrite4(FILE *fp, DWORD dwParam)
{
	FpWrite2(fp, (dwParam >> 16) & 0xffff);
	FpWrite2(fp, dwParam & 0xffff);
}

DWORD FpReadN(FILE *fp, int n)
{
	if (n == 1)
		return FpRead1(fp);
	else if (n == 2)
		return FpRead2(fp);
	else if (n == 3)
		return FpRead3(fp);
	else
		return FpRead4(fp);
}

void FpSeek(FILE *fp, DWORD n, int org)
{
	fseek(fp, n, org);
}

DWORD FpTell(FILE *fp)
{
	return ftell(fp);
}
