/*
 *  dvipr
 *      TeX Printer driver for The Canon LIPS III/IVc, CaPSL III printer
 *
 *      Developed at:
 *          Computer Systems Research Group
 *          Department of Information Science
 *          Fukui University
 *          9-1, Bunkyo-3, Fukui, 910 Japan
 *
 *      Authors:
 *          Yasumasa Koizumi, Atsushi Takagi, Kenichi Sugimoto,
 *          and Minato Kawaguti
 *
 *  The program may be copied and redistributed freely in the spirit of
 *  Free Software Foundation's "Copy Left", provided this title page is
 *  included in all of the source files involved.
 *
 *  Reports on porting to new platforms, bug fixes, etc., are highly
 *  appreciated. e-mail of this kind of communication shhould be addressedd
 *  to:
 *      dvipr@i1nws1.fukui-u.ac.jp
 */

#ifndef __FONT_H
#define __FONT_H

#include <stdio.h>
#include "newtypes.h"
#include "gvars.h"

#define	FONTNAME_MAX	256

#define	round8(n)	(((n) + 7) >> 3)

typedef enum {
	FONT_INTERNAL,
	FONT_DNPINTERNAL,
	FONT_INTERNAL_1BYTE,
	FONT_PK,
	FONT_DOWNLOAD,
	FONT_NONE
} FONTTYPE;

typedef struct tagGryph {
	unsigned DWORD	dwCharCode;
	unsigned DWORD	dwMoveX, dwMoveY;
	unsigned DWORD	dwWidth, dwHeight;
	DWORD			dwHorzOffset, dwVertOffset;
	DWORD		dwCounter;
	unsigned char* pBits;
	struct tagGryph* next;
} Gryph;

typedef struct tagCharWidth {
	unsigned DWORD	dwCharCode;
	DWORD			dwWidth;
	struct tagCharWidth* next;
} CharWidth;

typedef struct tagFonts {
	unsigned DWORD	dwFontNum;
	FONTTYPE		Type;
	BOOL			bDirection;	/* If true, is it a TateGaki(pTeX) font.*/
	char			acFilePath[FILENAME_MAX];
	char			acDir[FILENAME_MAX];
	char			acName[FONTNAME_MAX];
	unsigned DWORD	dwCheckSum;
	unsigned DWORD	dwScaled;
	unsigned DWORD	dwDesignSize;
	unsigned WORD	wDPI;
	Gryph*			pGryph;
	CharWidth*		pCharWidth;
	struct tagFonts* next;
} Fonts;

Fonts* pFontRoot;

BOOL SelectFont(Fonts* font);
void ResetCounter(void);
Fonts* GetFonts(unsigned DWORD fnum);
unsigned DWORD GetFontNumber(Fonts *pFont);
unsigned DWORD GetFontWidthMax(Fonts *pFont);
unsigned DWORD GetFontHeightMax(Fonts *pFont);
unsigned DWORD GetFontGryphs(Fonts *pFont);
Gryph* GetGryph(Fonts* font, unsigned DWORD code);
unsigned DWORD GetCharWidth(Fonts* font, unsigned DWORD code);
Fonts* CreateFont(void);
void CopyFont(Fonts *dest, Fonts *src);
Gryph* CreateGryph(Fonts* pFont);
CharWidth* CreateCharWidth(Fonts* pFont);
void DebugFontList(void);
BOOL RotateFont(Fonts *pFont);

#endif
